/**
 * 广告模块配置
 */
DGC.Ads = {

    /**
     * 页面配置
     */
    PAGES :{
        /**
         * 用户成就
         */
        UserAchieve: {
            LEFT: "4f5da34cef7759f964000000", //成就左
           RIGHT: "4f5da36bef77595464000000" //成就右
        },
        /**
         * 排行榜
         */
        BoardCategory:{
            LEFT: "4f5da385ef77590664000000", //SDK排行模式左
           RIGHT: "4f5da39aef77590664000001" //SDK排行模式右
        },
        /**
         * 排行榜
         */
        Dashboard:{
             LEFT: "4f5da3d5ef77590165000000", //SDK排行模式左
            RIGHT: "4f5da3e7ef7759f764000000" //SDK排行模式右
        },
        /**
         * 本地游戏
         */
        LocalGames:{
            BANNERS: "4f5da3f8ef77590265000000" //SDK游戏
        },

        /*
         * 玩过的游戏
         */
        PlayedGames:{
            BANNERS: "4f5da472ef77595865000000" //SDK我/玩家
        }
    },

    /**
     * 根据权重获取banner要展示的广告
     * @param {Json} params bannerAds 要显示的广告列表
     */
    getBannerShowAd:function(bannerAds){
        var r = Math.floor(Math.random() * 100+1);//产生1-100之间的随机数
        var offset = 0;
        for (var i=0; i < bannerAds.length; i++) {
              var bannerAd = bannerAds[i];
                  offset += bannerAd.percentage;
              if(r <= offset){
                return bannerAd;
              }
        }

        return bannerAds[0]; //默认
    },


    /**
     * 获取页面指定广告位ID
     * @param {Stirng} viewName 视图名称
     */
    getBanners: function(viewName){
        var viewBanners = DGC.Ads.PAGES[viewName],
            keys = [];
        for(var key in viewBanners){
            keys.push(viewBanners[key]);
        }
        return  keys;
    },

    /**
     * 获得左右区块HTML模板
     */
    getSidesAdTpl : function(){
        return '<div class="common-banner-ad hbox-layout" style="display:none;">\
                    <div class="box-flex-1"></div>\
                    <div class="ad-wrap" data-key="<*=this.leftAd.bid*>">\
                        <a href="#">\
                             <img src="<*=this.leftAd.img_url*>"></img>\
                        </a>\
                    </div>\
                    <div class="box-flex-1"></div>\
                    <div class="ad-wrap" data-key="<*=this.rightAd.bid*>">\
                        <a href="#">\
                            <img src="<*=this.rightAd.img_url*>"></img>\
                        </a>\
                    </div>\
                    <div class="box-flex-1"></div>\
                </div>'
    },

    /**
     * 获取滚动的模板
     */
    getScrollAdTpl : function(){
        return '<div class="common-ad-wrap">\
                    <div class="common-statebar">{0}</div>\
                    <div class="scroll-banner-ad">\
                        <div class="ad-wrap"><ul class="ad-content"></ul></div>\
                        <div class="ad-nav hbox-layout box-align-center box-pack-end">\
                            <div class="ad-nav-items hbox-layout box-align-center"> </div>\
                        </div>\
                        <div class="ad-title"></div>\
                   </div>\
                </div>';
    },

    /**
     * 滚动内容模板
     */
    getScrollAdCTpl:function(){
        return  '<li data-key="{0}" data-name="{1}"><img src="{2}"></img></li>';
    },

    /**
     * 滚动导航模板
     */
    getScrollAdNTpl:function(){
        return '<div class="ad-nav-item" data-key="{0}"></div>';
    }

}


/**
 * 广告Swipe组件
 */
DGC.Ads.SwipeAd = $.inherits(jQuery.Observer,{

    index: 0,

    duration: 300,

    constructor: function(element, options){
        DGC.Ads.SwipeAd.superclass.constructor.call(this);

        this.defineEvents(
            'swipeend', //切换广告完成事件
            'dragbegin' //手动切换广告事件
        )

        //可选参数
        this.options  = options || {};
        this.index = this.options.index || this.index;
        this.duration =  this.options.duration || this.duration;

         //注册事件
        if(this.options.events){
            for(var eventName in this.options.events){
                var eventCfg = this.options.events[eventName];
                this.on(eventName,eventCfg.fn , eventCfg.context);
            }
        }

        //DOM 节点
        this.containerEl = $(element);
        this.el = this.containerEl.children();
        this.ads = this.el.children();
        this.size = this.ads.size();

        //基本样式,初始化
        this.containerEl.css("overflow","hidden");
        this.el.css("listStyle",'none');

        if (this.size < 2){ return false;}

        this.width = this.containerEl.width();

        if (!this.width) return null;

        this.containerEl.css("visibility", "hidden");
        this.el.css({ width: this.size * this.width });
        this.ads.css({ width: this.width, display: "table-cell", verticalAlign:"top"} );

        this.swipe(this.index); //默认广告

        this.containerEl.css("visibility","visible");

        //监听容器事件
        this.el.bind("touchstart", $.bindFn(this.onTouchStart, this));
        this.el.bind("touchmove",  $.bindFn(this.onTouchMove, this) );
        this.el.bind("touchend", $.bindFn(this.onTouchEnd, this));
        this.el.bind("touchcancel",$.bindFn(this.onTouchEnd, this));
        this.el.bind("webkitTransitionEnd", $.bindFn(this.transitionEnd, this));
    },

    /**
     * 切换广告
     * @param {} index
     * @param {} duration
     */
    swipe: function(index, duration){
        var style = this.el[0].style;

        style.webkitTransitionDuration = style.transitionDuration = (duration || this.duration) + 'ms';
        style.webkitTransform = 'translate3d(' + -(index * this.width) + 'px,0,0)';

        this.index = index;
    },

    transitionEnd: function(e) {
        var context = this;  //多次触发transitionEnd
        clearTimeout(context.tTask);
        context.tTask = setTimeout(function(){
            context.fire("swipeend", context.index, context.ads[context.index]);
            context.tTask = null;
        }, 200);
    },

    onTouchStart: function(e) {
        this.start = {
           pageX: e.touches[0].pageX,
           pageY: e.touches[0].pageY,
           time: Number( new Date() )
        };

        this.isScrolling = null; //Y轴move动作操作
        this.deltaX = 0;

        this.el[0].style.webkitTransitionDuration = 0;

    },

    onTouchMove: function(e) {

        if(e.touches.length > 1 || e.scale && e.scale !== 1) return;

        this.deltaX = e.touches[0].pageX - this.start.pageX;  //水平位移

        if ( this.isScrolling == null) {  this.isScrolling = this.isMoveYAction(e);  }

        if (!this.isScrolling) {

          e.preventDefault();

          this.fire("dragbegin");

          this.el[0].style.webkitTransform = 'translate3d(' + (this.getMovePixel() - this.index * this.width) + 'px,0,0)';
        }

    },

    onTouchEnd: function(e) {
        var validSlide = this.isValidSlide(),  // determine if slide attempt triggers next/prev slide
            pastBounds = this.isPastBounds(); // determine if slide attempt is past start and end

        // if not scrolling vertically
        if (!this.isScrolling) {
          this.swipe( this.index + ( validSlide && !pastBounds ? (this.deltaX < 0 ? 1 : -1) : 0 ), this.duration );
        }
    },

   /**
    * 判断动作是否为Y轴移动
    * @param {} e
    * @return {}
    */
   isMoveYAction : function(e){
       return !!( this.isScrolling || Math.abs(this.deltaX) < Math.abs(e.touches[0].pageY - this.start.pageY) );
   },

   /**
    * 获得移动的X轴距离
    */
   getMovePixel: function(){
        //increase resistance if first or last slide
       return  this.deltaX /
          ( (!this.index && this.deltaX > 0               // if first slide and sliding left
            || this.index == this.size - 1              // or if last slide and sliding right
            && this.deltaX < 0                            // and if sliding at all
          ) ?
          ( Math.abs(this.deltaX) / this.width + 1 )      // determine resistance level
          : 1 );                                          // no resistance if false
   },

   /**
    * 固定时间内手指移动是否越过Swipe边界
    * @return {}
    */
   isValidSlide : function(){
       return  Number(new Date()) - this.start.time < 250      // if slide duration is less than 250ms
          && Math.abs(this.deltaX) > 20                   // and if slide amt is greater than 20px
          || Math.abs(this.deltaX) > this.width/2;        // or if slide amt is greater than half the width
   },

   /**
    * 切换是否为有效的Swipe
    * @return {}
    */
   isPastBounds: function(){
       return !this.index && this.deltaX > 0                       // if first slide and slide amt is greater than 0
          || this.index == this.size - 1 && this.deltaX < 0;    // or if last slide and slide amt is less than 0
   }

});


//命名空间
DGC.ViewPlugin = {};


/**
 * View左右分栏广告插件
 */
DGC.ViewPlugin.SildeAds = $.inherits($.emptyFn, {

    constructor: function(view){
        this.view = view;

        var prombanner = this.view.action.model["prombanner"];

        //手动注册回调
        prombanner.on("promotionbanner", this.loadAds, this);
        prombanner.on("localgamestate", this.showGameStatus, this);
    },

    /**
     * 启动插件
     */
    start: function(){
        //获取banner广告 SDK成就左,SDK成就右
        this.view.action.model["prombanner"].queryPromotionBanner(DGC.Ads.getBanners(this.view.alias));
    },

    /**
     * 销毁插件
     */
    destory: function(){
        var prombanner = this.view.action.model["prombanner"],
            gameModel = this.view.action.model["game"];

        prombanner.un("promotionbanner", this.loadAds); //手动取消监听回调
        prombanner.un("localgamestate", this.showGameStatus);

        this.waitLoadAds = null;
    },

    /**
     * 预加载广告图片信息
     */
    loadAds: function(banners){
        var context = this,
            keys = [];

        //待加载的 Ads
        context.waitLoadAds = {};

        //必须配备2个
        if(banners.length != 2){return false;}

        for(var i = 0; i < banners.length; i++){
            var banner = banners[i],
                selectAd = DGC.Ads.getBannerShowAd(banner.ads); //获得要渲染广告

            //需要请求的广告
            var requestAd = $.extend(selectAd,{
                "bid": banner["_id"], //广告位ID
                "url": selectAd.img_url //请求图片
            });

            //缓存广告信息
            context.waitLoadAds[requestAd.bid] = {
                status: false,
                data: requestAd
            }

            //FAKE对象
            keys.push(requestAd);
        }

        //预加载广告位图片
        context.view.loadImages(keys, function(img, data){
            context.waitLoadAds[data.bid].status = true; //广告位图片加载完毕

            context.sycPageAds(); //同步检查所有广告位是否加载成功
        });
    },

    /**
     * 同步检查广告加载
     */
    sycPageAds: function(){
        for( var key in this.waitLoadAds){
            var ad = this.waitLoadAds[key];
            if(!ad.status){
                return false; //尚未完成加载
            }
        }

        //加载完毕,渲染全部广告
        this.renderPromotionBanner();
    },

    /**
     * 渲染页面广告
     */
    renderPromotionBanner: function(){
         var PAGE_BID = DGC.Ads.PAGES[this.view.alias],
             loadedAds = this.waitLoadAds,
             tplObj = {
                leftAd: loadedAds[PAGE_BID.LEFT].data, //左边广告
                rightAd: loadedAds[PAGE_BID.RIGHT].data //右边广告
             },
             bannerModel = this.view.action.model['prombanner'];


        var renderBanner = $(this.view.renderByTpl(DGC.Ads.getSidesAdTpl(), tplObj));

        //点击触发
        renderBanner.find("a").bind("tap", {context : this}, function(event){
            var context = event.data.context;
                me = $(this),
                bid = me.parent(".ad-wrap").attr('data-key'),
                adObj = context.waitLoadAds[bid].data;
            switch(adObj.ad_type){
                case 1 /*APK*/:
                    DGC.ActionUtil.getGameState([adObj.package_name]);
                    context.cacheClickAd = adObj /*缓存点击广对象*/;
                    break;
                case 2 /*网页*/:
                    DGC.ActionUtil.openBrowser(adObj.ad_url);
                    break;
            }
        });

        this.view.el.prepend(renderBanner);
        renderBanner.fadeIn('slow');

        //统计广告位中显示次数
        setTimeout(function(){
            bannerModel.promotionBannerCount(tplObj.leftAd.id, tplObj.leftAd.bid);
            bannerModel.promotionBannerCount(tplObj.rightAd.id, tplObj.rightAd.bid);
        },0);

    },

    /**
     * 下载或启动游戏APK
     */
    showGameStatus : function(result) {
        var status = result.data.states[0],
            bannerModel = this.view.action.model["prombanner"];

        if(this.cacheClickAd){
            if(status) {
                bannerModel.promotionBannerClick(this.cacheClickAd.id, this.cacheClickAd.bid, 1/*运行*/ );//记录广告点击
                DGC.ActionUtil.startGame(this.cacheClickAd.package_name);//启动游戏
            } else {
               bannerModel.promotionBannerClick(this.cacheClickAd.id, this.cacheClickAd.bid, 0 /*下载*/);//记录广告点击
               DGC.ActionUtil.download(this.cacheClickAd.ad_url, this.cacheClickAd.package_name);//下载游戏
            }
            this.cacheClickAd = null;
        }
    }

});

/**
 * View滚动广告插件
 * @requires gameModel
 * @requires prombannerModel
 * @requires view
 */
DGC.ViewPlugin.ScrollAds = $.inherits(Object, {

    constructor: function(view){
        this.view = view;

        var prombanner = this.view.action.model["prombanner"];


        //手动注册回调
        prombanner.on("promotionbanner", this.loadAds, this);
        prombanner.on("localgamestate", this.showGameStatus, this);
    },

    /**
     * 启动插件
     */
    start: function(){
        this.view.action.model["prombanner"].queryPromotionBanner(DGC.Ads.getBanners(this.view.alias));
    },

    /**
     * 销毁插件
     */
    destory: function(){
        var prombanner = this.view.action.model["prombanner"];

        prombanner.un("promotionbanner", this.loadAds); //手动取消监听回调
        prombanner.un("localgamestate", this.showGameStatus);

        $(window).unbind('orientationchange', this.orientationListener);

        this.adEl = null;
        this.adIndexMap = null;
        this.waitLoadAds = null;
        this.renderIndex = null;

        //停止广告轮播
        clearTimeout(this.playTask);
    },

    /**
     * 预加载广告图片信息
     */
    loadAds: function(banners){
        var banner = banners[0], /*轮询只有一个广告位*/
            context = this,
            keys = [];

        //存在广告
        if(banner.ads.length > 0){
            context.cacheBanner = banner;
            //待加载标示
            context.waitLoadAds = {};
            context.adIndexMap = {};
            context.renderIndex = 0;


            //广告El Wrap
            context.adEl = $($.format(DGC.Ads.getScrollAdTpl(),[ resName.TEXT_AD_SCROLL_TITLE, banner._id]));
            if(!context.view.el) return false;
            context.view.el.prepend(context.adEl);

            for(var i = 0 ; i < banner.ads.length; i++){
                var ad = banner.ads[i];
                $.extend(ad, {
                    url: ad.img_url
                });

                //置标识位
                context.waitLoadAds[ad.img_url] = false;
            }

            //预加载广告位图片
            context.view.loadImages(context.cacheBanner.ads, function(img, data){
                if(!context.adEl) return false;

                var adCWrap = context.adEl.find(".ad-content"),
                    adNWrap = context.adEl.find(".ad-nav-items");

                adCWrap.append($.format(DGC.Ads.getScrollAdCTpl(),[ data.id,  data.ad_name, img.src ]));
                adNWrap.append($.format(DGC.Ads.getScrollAdNTpl(),[ data.id]));

                //置标识位
                context.waitLoadAds[data.img_url] = true;

                context.adIndexMap[data.id] = context.renderIndex++;

                context.sycPageAds(); //同步检查所有广告位是否加载成功
            });
        }
    },

    /**
     * 同步预加载图片
     */
    sycPageAds : function(){
         for( var key in this.waitLoadAds){
            if(!this.waitLoadAds[key]){
                return false; //尚未完成加载
            }
        }

        //加载完毕,渲染全部广告
        this.renderPromotionBanner();
    },

    //渲染广告
    renderPromotionBanner: function(result){
        var adEl = this.adEl,
            context = this;

        //点击统计
        adEl.find("li").bind('tap', {context : this}, function(event){
            var context = event.data.context,
                me = $(this),
                id = me.attr('data-key'),
                ads = context.cacheBanner.ads,
                adObj;

                for(var i = 0 ; i < ads.length; i++){
                   adObj = ads[i];
                   //找到匹配的广告对象
                   if(adObj.id == id){
                       break;
                   }
                }
            switch(adObj.ad_type){
                case 1 /*APK*/:
                    DGC.ActionUtil.getGameState([adObj.package_name]);
                    context.cacheClickAd = adObj /*缓存点击广对象*/;
                    break;
                case 2 /*网页*/:
                    DGC.ActionUtil.openBrowser(adObj.ad_url);
                    break;
            }
        });


        if(context.cacheBanner.ads.length > 1){
            //创建广告轮播组件
            context.swipeAd = new DGC.Ads.SwipeAd(adEl.find(".ad-wrap"),{
                index: context.radomADIndex(), //默认设置轮播初始位置
                events:{
                    "swipeend": {
                        fn: context.highLightAD, //通知当前选中
                        context: context
                    },
                    "dragbegin":{
                        fn: context.pausePlay, //通知当前开始手动切换
                        context: context
                    }
                }
            });
        }else{
            context.highLightAD(0 /*默认值*/, adEl.find("li:first"));
        }


        context.orientationListener = $.bindFn(context.switchAdStatus, context);
        //监听屏幕切换
        $(window).bind('orientationchange', context.orientationListener);
        //判断当期旋屏状态是否显示Banner
        setTimeout(context.orientationListener, 0);
    },

    /**
     * 切换完成,高亮当前广告回调
     * @param {Number} index
     * @param {jQuery Element} itemEl
     */
    highLightAD: function(index, itemEl){
        var el = $(itemEl);

        //高亮
        this.adEl.find(".ad-title").text(el.attr("data-name"));
        this.adEl.find(".active").removeClass("active");
        this.adEl.find(".ad-nav-item[data-key='"+ el.attr("data-key") +"']").addClass("active");

        //启动下一个
        this.nextAD();
    },

    /**
     * 暂停轮播
     */
    pausePlay: function(){
       clearTimeout(this.playTask);
       this.playTask = null;
    },

    /**
     * 间隔时间后启动任务
     */
    nextAD: function(){
        var selfFn = arguments.callee,
            context = this;

        //如果广告少于2个，则固定显示
        if(context.cacheBanner.ads.length < 2) return false;

        clearTimeout(context.playTask);
        context.playTask = setTimeout( function(){
            var selectIndex = context.radomADIndex(); //按概率选取图片
            if(selectIndex != context.swipeAd.index){ //非正在显示广告
               context.swipeAd.swipe(selectIndex);
            }else{
               selfFn.call(context);
            }
        }, context.cacheBanner.duration * 1000);
    },


    /**
     * 获得随机广告的索引
     */
    radomADIndex : function(){
        var ad = DGC.Ads.getBannerShowAd(this.cacheBanner.ads),
            context = this;

        //统计广告位中被选取次数
        setTimeout(function(){
           context.view.action.model["prombanner"].promotionBannerCount(ad.id, context.cacheBanner._id);
        }, 0);

        return this.adIndexMap[ad.id];
    },

    /**
     * 切换显示广告状态
     */
    switchAdStatus: function(){
        var orientation = window.orientation,
           context = this,
           adEl = this.adEl;

        if(DEBUG.isWebModel()){
            orientation = 0;
        }

        switch(orientation) {
            case 0:
            case 180:  /*横屏*/
                adEl.show( 300,  function(){
                    context.nextAD();
                });
                break;
            case 90:
            case -90:  /*竖屏*/
                context.pausePlay();
                adEl.hide();
                break;
        }
    },

    /**
     * 下载或启动游戏APK
     */
    showGameStatus : function(result) {
        var status = result.data.states[0],
            bannerModel = this.view.action.model["prombanner"];

        if(this.cacheClickAd){
            if(status) {
                bannerModel.promotionBannerClick(this.cacheClickAd.id, this.cacheBanner._id, 1/*运行*/ );//记录广告点击
                DGC.ActionUtil.startGame(this.cacheClickAd.package_name);//启动游戏
            } else {
               bannerModel.promotionBannerClick(this.cacheClickAd.id, this.cacheBanner._id, 0 /*下载*/);//记录广告点击
               DGC.ActionUtil.download(this.cacheClickAd.ad_url, this.cacheClickAd.package_name);//下载游戏
            }
            this.cacheClickAd = null;
        }
    }
});