/**
 * 自定义窗口父类
 */
DGC.Dialog = $.inherits(DGC.UIBase,{

    /**
     * 不自动显示
     */
    autoShow: false,

    /**
     * 默认渲染位置
     * @type
     */
    renderTo : 'body',

    isDeferRender: true,

    isScrollable: false,

    maskTransparent: false,

    /**
     * 渲染模板
     * @type String url加载
     */
    renderTpl : "<div class='common-dialog' style='display:none;'><div class='header'></div><div class='body'></div></div>",

    viewReady: function(){},

    /**
     * 点击取消(关闭),back键，需要执行的函数
     * @type
     */
    closeAction: $.emptyFn,

    addMask: function(){
        this.mask = $($.format("<div class='common-mask common-dialog-mask' style='display:none;' id='{0}'></div>", [this.baseId+"-mask"]));
        if(this.maskTransparent){
            this.mask.removeClass("common-dialog-mask");
        }
        this.mask.insertBefore(this.el);
        this.resetMask();

        this.resetFn  = $.bindFn(this.resizePostion, this);
        $(window).bind('orientationchange', this.resetFn );
    },

    /**
     * WebKit browser: Input elements disregard z-index and erroneously steal focus
     * http://code.google.com/p/android/issues/detail?id=6721
     * @param {Boolean} enbale
     */
    _lockView: function(enbale){
        var cView = DGC.Application.service.currentModule.currentView;
        if(cView && cView.el && cView.isDeferRender){
            if(enbale){
                cView.el.find("input, select, textarea").attr('disabled','disabled')
                cView.el.find("a").css("visibility","hidden");
            }else{
                cView.el.find("input, select, textarea").removeAttr('disabled');
                setTimeout(function(){
                    if(cView.el){
                        cView.el.find("a").css("visibility","visible");
                    }
                },100)

            }
        }
    },

    resizePostion: function(){
        var context =  this;
        clearTimeout(context.delayResizeTask);
        this.delayResizeTask = setTimeout(function(){
            context.resetMask();
            context.resetPos();
        }, 200);
    },

    /**
     * 重设Mask高度
     */
    resetMask: function(){
        var context = this;
        if(document.body){
            context.mask.css({
                "height": $(document).height()
            });
        }
    },

    resetPos: function(){
         //设置垂直居中
        if($(window).height() > this.el.height()){
            this.el.css({
                "top": $(window).scrollTop() + ($(window).height() - this.el.height()) / 2
            });
        }else{
             this.el.css({
                 "top": this.scrollPostion
             });
        }
    },

    setzIndex: function(){
        //获取当前最大z-index
        var maxZ = Math.max.apply(null,$.map($('body > *'), function(e,n){
           if($(e).css('position')=='absolute')
                return parseInt($(e).css('z-index'))||1 ;
           })
    	);
        //设置z-index
        this.mask.css("z-index", maxZ + 1000 );
        this.el.css("z-index", maxZ + 1200);
    },

    onHide : function(){
        this.mask.hide();
    },

    onShow: function(){
        var zMasks = DGC.Dialog.ManagerUtil.zMasks;
        this.mask.show();

        //清除当前页面焦点
        $("input").each(function(index, dom) {
            dom.blur();
        });

        DGC.ActionUtil.clearFocus();

        //当前没有窗口，则调用Mask显示
        if(zMasks.length == 0 ){
            DGC.ActionUtil.setShade(false);
            //Hacker Android Web block input/select/textarea
            this._lockView(true);
        }

        //存储当前Dialog表示存在
        zMasks.push(this);
    },

    onRender : function(){

        this.setLocalString();

        DGC.Dialog.superclass.onRender.call(this);

        this.addMask();

        //点击关闭/Back键执行函数
        this.closeProxyFn = $.bindFn(function(){
            var context = this;
            setTimeout( function(){
                context.destory();
                context.closeAction();
            } , 500 );
        }, this);

        this.viewReady();

        this.setzIndex();

        //缓存第一次显示的
        this.scrollPostion = $(window).scrollTop();

        //设置垂直居中
        if($(window).height() > this.el.height()){
            this.el.css({
                "top": $(window).scrollTop() + ($(window).height() - this.el.height()) / 2
            });
        }
    },

    onInitialize :function(){
        DGC.Dialog.ManagerUtil.add(this.baseId , this);
        DGC.Dialog.superclass.onInitialize.call(this);
    },

    onDestory: function(){
        var zMasks = DGC.Dialog.ManagerUtil.zMasks;

        clearTimeout(this.delayResizeTask);
        $(window).unbind('orientationchange', this.resetFn);
        this.resetFn = null;

        this.mask.remove();
        this.mask = null;
        zMasks.pop();

        //当前没有Mask层则，隐藏Shade
        if(zMasks.length == 0 ){
            DGC.ActionUtil.setShade(true);
            this._lockView(false);
        }
        DGC.Dialog.ManagerUtil.remove(this.baseId);
    }
});

/**
 * 窗口管理类
 */
DGC.Dialog.ManagerUtil = {

    /**
     * 当前viewsMask层叠
     */
    zMasks: [],
    /**
     * 缓存的弹出窗口对象
     */
    caches: {},

    /**
     * 缓存对象
     * @param {String} index alertID
     * @param {Object} instance 实例对象
     */
    add: function(id, instance){
        this.caches[id] = instance;
    },

    /**
     * 移除对象
     * @param {String} index alertID
     */
    remove: function(id){
        this.caches[id] = null;
        delete this.caches[id];
    },

    /**
     * 判断相同字符串提示是否存在
     * @param {String} error 提示字符串
     */
    isSameMsgShow: function(error){
        for(var key in this.caches){
            var instance = this.caches[key];
            DGC.ActionUtil.log("text:" + instance.text);
            if(instance.text == error){
                return true;
            }
        }
        return false;
    },

    /**
     * 销毁当前显示窗口
     */
    popzIndexDialog: function(){
        if(this.zMasks.length > 0 ){
            var dialog = this.zMasks.pop();
            dialog.closeProxyFn();
            return true;
        }

        return false;
    }

}


/**
 * 消息提示框
 */
DGC.Alert = $.inherits(DGC.Dialog,{

    baseId: "alert-win",

    text: undefined,

    callback: undefined,

    viewReady: function(){
        //设置内容
        var contentEl = this.el.find(".content"),
            okEl = this.el.find(".btn-ok"),
            DC  = DGC.Common;

        contentEl.html(this.text);
        okEl.text(resName.BUTTON_OK);

        //绑定事件
        okEl.bind(DC.E_TAP_DOWN, DC.buttonTapDownHandler);
        okEl.bind(DC.E_TAP_UP, DC.buttonTapEndHandler);

        //确认(Close)按钮
        okEl.bind("tap", this.closeProxyFn);

        //设置显示位置
        this.el.css( {"top": document.body.scrollTop} );
    },

    /**
     * 关闭窗口时，执行回调
     */
    closeAction: function(){
        this.callback = this.callback || $.emptyFn;
        this.callback.call(this);
    },

    onInitialize :function(){
        //获得配置模板路径
        this.renderTpl = QueenConfig.dialogConfig['Alert'].tpl;

        DGC.Alert.ManagerUtil.add(this.id , this);
        DGC.Alert.superclass.onInitialize.call(this);
    },

    onDestory : function(){
        DGC.Alert.superclass.onDestory.call(this);
        DGC.Alert.ManagerUtil.remove(this.id);
    }
});

/**
 * 弹出框工具类
 */
DGC.Alert.ManagerUtil = {

    /**
     * 缓存的弹出窗口对象
     */
    caches: {},

    /**
     * 缓存对象
     * @param {String} index alertID
     * @param {Object} instance 实例对象
     */
    add: function(id, instance){
        this.caches[id] = instance;
    },

    /**
     * 移除对象
     * @param {String} index alertID
     */
    remove: function(id){
        this.caches[id] = null;
        delete this.caches[id];
    },

    /**
     * 判断相同字符串提示是否存在
     * @param {String} error 提示字符串
     */
    isSameMsgShow: function(error){
        for(var key in this.caches){
            var instance = this.caches[key];
            DGC.ActionUtil.log("text:" + instance.text);
            if(instance.text == error){
                return true;
            }
        }
        return false;
    }
}

/**
 * 确认对话框
 */
DGC.Confirm = $.inherits(DGC.Dialog,{

    baseId: "confirm-win",

    title: undefined,

    text: undefined,

    callback: $.emptyFn,

    onInitialize: function(){
        //获得配置模板路径
        this.renderTpl = QueenConfig.dialogConfig['Confirm'].tpl;

        DGC.Confirm.superclass.onInitialize.call(this);
    },

    viewReady: function(){

        //设置内容
        this.el.find(".header h1").text(this.title);

        var contentEl = this.el.find(".content"),
            okEl = this.el.find(".btn-ok"),
            cancelEl = this.el.find(".btn-cancel");


        if($.isString(this.text)){
            contentEl.text(this.text);
            okEl.text(resName.BUTTON_OK);
            cancelEl.text(resName.BUTTON_CANCEL);
        }else{
            contentEl.text(this.text.content);
            okEl.text(this.text.ok_text || resName.BUTTON_OK);
            cancelEl.text(this.text.cancel_text ||resName.BUTTON_CANCEL);
        }

        //绑定事件
        var btns = this.el.find(".btn");
        btns.bind(DGC.Common.E_TAP_DOWN,function(){
            $(this).addClass('active');
        });
        btns.bind(DGC.Common.E_TAP_UP,function(){
            $(this).removeClass('active');
        });

        //确认按钮
        this.el.find(".btn-ok").bind("tap", { context: this }, function(event){
            var context = event.data.context;
            setTimeout(function(){
                context.destory();
                context.callback(true);
            } , 500 );
        });

        //取消按钮
        this.el.find(".btn-cancel").bind("tap",this.closeProxyFn);

    },

    /**
     * 关闭窗口时，执行回调
     */
    closeAction: function(){
        this.callback = this.callback || $.emptyFn;
        this.callback(false);
    }

});

/**
 * 输入对话框
 */
DGC.Prompt = $.inherits(DGC.Dialog, {

    baseId: "prompt-win",

    /**
     * 对话框
     */
    title: undefined,

    /**
     * 默认属性
     */
    inputAttrs:undefined,

    /**
     * 回调
     */
    callback: undefined,

    onInitialize: function(){
        //获得配置模板路径
        this.renderTpl = QueenConfig.dialogConfig['Prompt'].tpl;
        DGC.Prompt.superclass.onInitialize.call(this);
    },

    viewReady : function(){

        //设置内容
        this.el.find(".header h1").text(this.title);

        //默认显示值
        if(this.inputAttrs){
            this.el.find("input").attr(this.inputAttrs);
        }

        //确认按钮
        this.el.find(".btn-ok").bind("tap", { context: this }, function(event){
            var context = event.data.context,
                val  = context.el.find("input").val();
            setTimeout(function(){
                context.destory();
                context.callback(true, val);
            } , 500 );
        });

        //取消按钮
        this.el.find(".btn-cancel").bind("tap", this.closeProxyFn);
    },

    /**
     * 关闭窗口时，执行回调
     */
    closeAction: function(){
        this.callback = this.callback || $.emptyFn;
        this.callback(false);
    }
});
