/**
 * 开发者购买自定义商品
 */
DGC.View.DeveloperPayUser =  $.inherits(DGC.View, {

    baseId: 'nativepayuser-view',

    /**
     * 剩余乐币
     */
    playerCoins: undefined,

    /**
     * 是否支持乐币
     */
    enableCoins: undefined,

    /**
     * 订单信息
     */
    orderInfo: undefined,

    /**
     * 道具类型自定义道具类型
     * 0:乐币
     * 1:注册道具
     * 2:自定义道具
     * 3:网游货币
     * 4:CheckPoint
     * 5:CheckPoint Group
     * @type Number
     */
    type: 1 ,

    viewReady: function(){
        DGC.ActionUtil.fullScreen(true);
        //DGC.ActionUtil.transparentBG();

        //显示窗口需要的数据源
        this.requestAPI = {
            orderReady: true //道具信息
        };

        //支持乐币支付，非离线模式则查询当前余额
        if(DH.isSupportCoins() && !window.sdk.isOffline && this.type != 5 /*非CheckPoint group类型*/){
            $.extend(this.requestAPI, { coinsReady: false });
            this.action.model['pay'].queryPlayerCoins( false );
        }else{
            this.directToBuy();
        }
    },

    /**
     * 保存用户剩余乐币数
     * @param {Number} coins 乐币数
     * @param {Boolean} enable 是否支持乐币
     */
    cachePlayerCoins: function(coins){
        //乐币数
        this.playerCoins = window.sdk.player.coins = coins;
        $.extend(this.requestAPI, { coinsReady: true });

        this.directToBuy();
    },

    /**
     * 判断所有请求是否都已经响应
     */
    isAPIReady : function(){
        for(var key in this.requestAPI){
            if(!this.requestAPI[key]){
                return false;
            }
        }
        return true;
    },

    /**
     * 判断根据乐币和道具信息支付方式
     */
    directToBuy: function(){
        //未加载完成，无需判断
        if(!this.isAPIReady()){
            return false;
        }

        var quantity = this.orderInfo.quantity || 1 /*默认开发者入口只能购买一个道具*/,
            context = this;

        //防止开发者不给数量值
        $.extend(this.orderInfo, { "quantity": quantity } );

        context.buyAction();
    },

    /**
     * 购买道具逻辑
     */
    buyAction: function(){
        //道具列表
        if(window.sdk.isOffline){
            this.action.setView("OfflineCharge",{
                orderInfo: this.orderInfo
            }, false /*不重置视图*/, true /*不保存当前视图*/);
        }else {
             switch(this.type){
                case 1 /*普通道具*/:
                case 2 /*非注册道具*/:
                    //启用乐币支付且乐币足够
                    if(DH.isSupportCoins() && this.isEnoughCoins() ){
                        this.action.model['pay'].executePay(0 /*乐币支付*/, {
                            orderInfo: this.orderInfo,
                            product_type: this.type
                        });
                    }else{
                        this.switchToChargeList();
                    }
                    break;
                case 4 /*CheckPoint*/:
                    //启用乐币支付且乐币足够
                    if(DH.isSupportCoins() && this.isEnoughCoins() ){
                        this.action.model['pay'].executePay(10 /*乐币支付*/, {
                            orderInfo: this.orderInfo,
                            product_type: this.type
                        });
                    }else{
                        this.switchToChargeList();
                    }
                    break;
                case 5 /*CheckPoint Group*/:
                    this.switchToChargeList(); //仅支付RMB支付
                    break;
            }
        }
    },

    /**
     * 切换到充值/支付列表
     */
    switchToChargeList : function(){
        //进入支付列表
        this.action.setView("ChargeAccount",{
            chargeModel: 1 /*开发者界面进入支付列表*/,
            orderInfo: this.orderInfo,
            type: this.type
        }, false /*不重置视图*/, true /*不保存当前视图*/);
    },

    /**
     * 判断当前乐币是否能购买道具
     * 乐币支持下面三种道具类型
     */
    isEnoughCoins : function(){
        var price;
        switch(this.type){
           case 1 /*注册道具*/:
           case 2 /*非注册道具*/:
               price = this.orderInfo.item.product.price;
               break;
           case 4 /*CheckPoint*/:
               price = this.orderInfo.price;
               break;
        }

        return this.playerCoins >= (price * this.orderInfo.quantity);
    },

    /**
     * 通知SDK更新游戏道具信息
     * @param {Object} payResult 支付结果
     * @param {Object} detail 支付结果详情
     * @param {Object} extras extras对象
     * */
    showLedouPayResult : function(result, detail , extras ){
        this.action.model['pay'].ledouOrderNotify({
            result:result,
            type: this.type ,
            detail: detail,
            extras: extras,
             //提示自定义信息后，返回
            success: this.returnToGameAction,
            fail: this.returnToGameAction
        });
    },

    /**
     * 当该页面请求超时，点击确认，直接返回游戏
     */
    returnToGameAction: function(){
        //返回游戏
        DGC.ActionUtil.returnToGame();
    }

});