/**
 * 用户当前成就信息
 */
DGC.View.UserAchieve =  $.inherits(DGC.View, {

    baseId: 'user-achievement',

    /**
     * 别名
     */
    alias: 'UserAchieve',

    /**
     * 根视图
     */
    isSub: false,

    game: undefined,

    player: undefined,

    /**
     * 视图显示模式
     * 0:SDK页面
     * 1:游戏开发者界面
     */
    model: 0,

    /**
     * 视图初始化函数
     */
    viewReady: function(){

        this.viewTpl =  {
            "ACHIEVE" : "#ua-listitem-tpl"
        };

        //如果为离线模式，则查询本地成就
        if(window.sdk.isOffline){
            this.setTitle($.format(resName.A_TITLE_ACHIEVE , [window.sdk.game_name] ));
            this.action.model['achi'].queryOfflineGameAchieve();
            this.bindReloginBtn();
        }else{
            var gameName = (this.game != null ? this.game.name : window.sdk.game.name);
            this.setTitle($.format(resName.A_TITLE_ACHIEVE , [ gameName] ));
            //默认加载请求"我"的
            var gameId =  this.game ? this.game.id : window.sdk.game.id,
                playerId = this.player ? this.player.id : window.sdk.player.id;

            this.action.model["achi"].queryPlayerGameAchieve(gameId, playerId);

             //广告显示
            this.adPlugin = new DGC.ViewPlugin.SildeAds(this);
            this.adPlugin.start();
        }

        //记录页面访问次数
        DGC.ActionUtil.recordPageCount(DGC.Event.PageId.SKYNET_PAGE_ACHIEVEMENT);
    },


    /**
     * 离线重登录按钮绑定
     */
    bindReloginBtn : function(){
         $("#offline-relogin").bind(DGC.Common.E_TAP_DOWN,function(){$(this).addClass('active')})
            .bind(DGC.Common.E_TAP_UP,function(){$(this).removeClass('active')})
            .bind("tap", { context: this } , function(event){
                DGC.ActionUtil.offlineRelogin();
            });
         $("#ua-offline-tip").show();
    },

    /**
     * 渲染用户在游戏中获取的成就
     */
    renderAchieveList : function(list){
        var doneAchieve = 0 ,
            achieve,
            item,
            item_achieve,
            renderList = [],
            drawList =[];

        for (var i = 0; i < list.length; i++) {
           item = list[i], item_achieve = item.achievement;

           //统计完成的成就数
           if(item.percentage == 1){
               doneAchieve++;
           }

           //转换对象
           achieve = {
               id: item_achieve.id,
               title: item_achieve.title,
               icon: item_achieve.icon_url,
               desc: this.getDescText(item),
               percentText: this.getPercentText(item.percentage),
               percent: item.percentage
           }

           //存在图标，且需要绘制的成就图标
           if(!$.isEmpty(achieve.icon) && item.percentage > 0 && item.percentage < 1){
               drawList.push($.extend(achieve,{url : achieve.icon}));
           }

           renderList.push(achieve);
       }

       //渲染成就HTML结构
       var renderItems = $(this.renderByTpl(this.viewTpl.ACHIEVE, renderList));
       renderItems.bind(DGC.Common.E_TAP_DOWN,DGC.Common.listTapDownHandler);
       renderItems.bind(DGC.Common.E_TAP_UP,DGC.Common.listTapEndHandler);
       this.el.find(".achievement-list").html(renderItems);

       //设置标题
       $("#ua-achieve-done .achive-desc").text( (doneAchieve == 0 ? resName.TEXT_USERACHIEVE_NOT_GET_ACHIEVE
               : $.format(resName.TEXT_USERACHIEVE_GOT_ACHIEVE, [doneAchieve])  ));

       $("#ua-achieve-done .achive-num").text(doneAchieve + "/" +  list.length);

       //绘制进行中的成就
       this.renderProgressIcon(drawList);
    },

    /**
     * 绘制正在进行的成就图标
     */
    renderProgressIcon: function(achieves){
        var me = this;
        this.loadImages(achieves, function(img, data){
            var id = "icon-progress-"+ data.id,
                percent = data.percent;
            //加载图片完成后，绘制当期完成度
            me.drawPrecent(id, img, percent);
        });
    },

    /**
     * 获得成就描述信息
     */
    getDescText: function(item){
        var achieve = item.achievement,
            desc;

        //已完成成就，则直接显示获得后的信息
        if(item.percentage == 1){
            desc = achieve.earned_desc;
        }else{
            desc = (achieve.is_hidden ? resName.TEXT_USERACHIEVE_HIDDEN_NOT_GET :
                  achieve.pre_earned_desc );
        }

        //未完成状态
        return $.isEmpty(desc) ? "" : desc;
    },

    /**
     * 获取显示百分比的文字
     */
    getPercentText : function(percent){
        var text;
        switch(percent){
            case 0:
                text = resName.TEXT_USERACHIEVE_NOT_GET;
                break;
            case 1:
                text = $.format("<span class='get-achieve-text'>{0}</span>",[resName.TEXT_USERACHIEVE_GOT]);
                break;
            default:
                text = $.format(resName.TEXT_USERACHIEVE_PRECENT,[ parseInt(percent * 100, 10) ]);
                break;
        }
        return text;
    },

    /**
     * 根据完成度显示成就图标
     * @param {String} canvasID 画布ID
     * @param {Image}  img      已经加载图标完毕的image对象
     * @param {Number} percent  完成百分比,如:0.76
     */
    drawPrecent : function(canvasID , img,  percent){
        var canvas = document.getElementById(canvasID),
            ctx = canvas.getContext('2d'),
            cx = 35,
            cy = 35,
            cw = 70,
            ch = 70,
            radius  = 35,
            startAngle = 1.5 * Math.PI,
            endAngle = startAngle + ( percent * 360)  * Math.PI / 180 ;

        //设置画布大小
        canvas.width = cw;
        canvas.height = ch;
        //Android2.0/2.1分辨率缩放存在缩放BUG，图片将绘制到画布外
        //问题BUG:http://code.google.com/p/android/issues/detail?id=5141
        //解决方案参考：http://trac.osgeo.org/openlayers/changeset/11868
        var factor = /android 2.0|android 2.1/.test(navigator.userAgent.toLowerCase()) ?
                320 / window.screen.width : 1;
        ctx.drawImage(img, 0, 0, cw * factor, ch * factor ); //绘制成就图标
        this.drawSector(ctx, cx, cy, startAngle , endAngle , radius + 1, true); //逆时针绘画完成度
    },

    /**
     * 绘制扇形
     * @param {Canvas} ctx 画布Context
     * @param {Number} cx  中心坐标点x
     * @param {Number} cy  中心坐标点y
     * @param {Number} start_angle 开始弧度
     * @param {Number} end_angle   结束弧度
     * @param {Number} radius      半径
     * @param {Boolean} anticlockwise 顺/逆时针 false:顺时针  true:逆时针
     */
    drawSector : function(ctx ,cx, cy, start_angle, end_angle, radius, anticlockwise){
         start_angle = start_angle || 0;
         //开始绘制路径
         ctx.beginPath();
         //画出弧线
         ctx.arc(cx, cy, radius, start_angle, end_angle, anticlockwise);
         //画出结束半径
         ctx.lineTo(cx, cy);
         ctx.fillStyle = '#FFFFFF';
         ctx.fill();
    },

    /**
     * 销毁广告
     */
    onDestory: function(){
        if(this.adPlugin){
            this.adPlugin.destory();
            this.adPlugin = null;
        }
    }

});
